# import json
# import logging
# import os
# from mind_castle.secret_store_base import SecretStoreBase

# logger = logging.getLogger(__name__)


# class JsonSecretStore(SecretStoreBase):
#     """
#     An json-file secret store.
#     Use only for testing or development.
#     """

#     store_type = "json"
#     filename = "secrets.json"

#     def __init__(self):
#         if not os.path.exists(self.filename):
#             with open(self.filename, "w") as f:
#                 f.write("{}")

#     def create_secret(self, key: str, value: str) -> dict:
#         with open(self.filename, "r") as f:
#             secrets = json.load(f)
#             secrets[key] = value

#         with open(self.filename, "w") as f:
#             json.dump(secrets, f)
#         return {"mind_castle_secret_type": self.store_type, "key": key}

#     def retrieve_secret(self, secret_details: dict) -> str:
#         key = secret_details.get("key")
#         with open(self.filename, "r") as f:
#             secrets = json.load(f)
#         return secrets.get(key)

#     def update_secret(self, secret_details: dict, value: str) -> dict:
#         return self.create_secret(secret_details.get("key"), value)

#     def delete_secret(self, secret_details: dict) -> None:
#         key = secret_details.get("key")
#         with open(self.filename, "r") as f:
#             secrets = json.load(f)

#         del secrets[key]

#         with open(self.filename, "w") as f:
#             json.dump(secrets, f)



from rich.table import Table
from rich import print
from rich.console import Group
from rich.panel import Panel
from rich.text import Text

from mind_castle import stores  # noqa: F401
from mind_castle.secret_store_base import SecretStoreBase


header = Text("\n[bold underline]Mind Castle[/bold underline]\n", justify="center")

table = Table(title="Secret Stores")
table.add_column("Store Type", justify="left", style="cyan bold")
table.add_column("Required env var", justify="left", style="magenta")
table.add_column("Optional env var", justify="left", style="green")

for store in SecretStoreBase.__subclasses__():
    for config_set in range(len(store.required_config)):
        if config_set > 0:
            table.add_row("", "--- OR ---", "")
        table.add_row(
            store.store_type if config_set == 0 else "",
            "\n".join(store.required_config[config_set]),
            "\n".join(store.optional_config),
        )
    table.add_row("", "", "")

print(Panel.fit(Group(Text(), table), title="MIND CASTLE"))



import os
import logging
import base64
from cryptography.hazmat.primitives.ciphers.aead import AESGCM

from mind_castle.secret_store_base import SecretStoreBase, MindCastleSecret

logger = logging.getLogger(__name__)


class LocalEncryptionSecretStore(SecretStoreBase):
    """
    Uses a local secret to encrypt and store cyphertext in the DB.
    Local encryption key must be 128, 192, or 256 bits
    """

    store_type = "localencryption"
    required_config = [
        [
            "MIND_CASTLE_LOCAL_ENCRYPTION_SECRET",
        ]
    ]
    optional_config = []

    def __init__(self):
        self.aesgcm = AESGCM(
            base64.urlsafe_b64decode(
                os.environ.get("MIND_CASTLE_LOCAL_ENCRYPTION_SECRET")
            )
        )

    def retrieve_secret(self, secret: MindCastleSecret) -> str:
        blob = base64.urlsafe_b64decode(secret.encrypted_value)
        nonce, ct = blob[:12], blob[12:]
        return self.aesgcm.decrypt(nonce, ct, None).decode()

    def create_secret(self, value: str) -> MindCastleSecret:
        nonce = os.urandom(12)
        ciphertext = self.aesgcm.encrypt(nonce, value.encode(), None)
        return MindCastleSecret(
            mind_castle_secret_type=self.store_type,
            encrypted_value=base64.b64encode(nonce + ciphertext).decode(),
        )

    def update_secret(self, secret: MindCastleSecret, value: str) -> MindCastleSecret:
        new_secret = self.create_secret(value)
        new_secret.key = secret.key
        return new_secret

    def delete_secret(self, secret: MindCastleSecret) -> None:
        pass



# from unittest.mock import ANY

# from mind_castle.secret_store_base import SecretStoreBase
# from mind_castle.stores.vault import HashiCorpVaultSecretStore


# def test_put_secret(mocker):
#     secret_store = HashiCorpVaultSecretStore()
#     key = secret_store.get_secret_key()
#     mocker.patch.object(
#         secret_store.client.secrets.kv.v1, "create_or_update_secret", return_value={}
#     )
#     response = secret_store.put_secret(key, "some_secret_value")

#     assert secret_store.client.secrets.kv.v1.create_or_update_secret.called_once_with(
#         secret=dict(path=ANY, secret_value="some_secret_value")
#     )
#     assert response["mind_castle_secret_type"] == secret_store.store_type
#     assert response["key"] is not None


# def test_get_secret(mocker):
#     mock_vault_response = {
#         "data": {
#             "data": {
#                 "secret_value": "some_secret_value",
#             }
#         }
#     }

#     secret_store = HashiCorpVaultSecretStore()
#     mocker.patch.object(
#         secret_store.client.secrets.kv.v1,
#         "read_secret",
#         return_value=mock_vault_response,
#     )
#     response = secret_store.get_secret(
#         {"mind_castle_secret_type": "hashicorpvault", "key": "some_secret_key2"}
#     )

#     assert secret_store.client.secrets.kv.v1.read_secret.called_once_with(
#         path="some_secret_key2"
#     )
#     assert response == "some_secret_value"


# def test_delete_secret(mocker):
#     secret_store = HashiCorpVaultSecretStore()
#     mocker.patch.object(
#         secret_store.client.secrets.kv.v1, "delete_secret", return_value={}
#     )

#     secret_store.delete_secret({"mind_castle_secret_type": "hashicorpvault", "key": "some_secret_key"})
#     assert secret_store.client.secrets.kv.v1.delete_secret.called_once_with(
#         path="some_secret_key"
#     )



# import logging

# from mind_castle.secret_store_base import SecretStoreBase, MindCastleSecret

# logger = logging.getLogger(__name__)


# class MemorySecretStore(SecretStoreBase):
#     """
#     An in-memory secret store.
#     This is not persistent and will be lost when the application is restarted.
#     Use only for testing or development.
#     """

#     store_type = "memory"

#     def __init__(self):
#         self.secrets = {}

#     def create_secret(self, value: str) -> MindCastleSecret:
#         secret = MindCastleSecret(
#             mind_castle_secret_type=self.store_type,
#             encrypted_value=value,
#         )
#         self.secrets[secret.key] = secret
#         return secret

#     def retrieve_secret(self, secret: MindCastleSecret) -> str:
#         return self.secrets.get(secret.key).encrypted_value

#     def update_secret(self, secret: MindCastleSecret, value: str) -> MindCastleSecret:
#         new_secret = self.create_secret(value)
#         new_secret.key = secret.key
#         self.secrets[new_secret.key] = new_secret
#         return new_secret

#     def delete_secret(self, secret: MindCastleSecret) -> None:
#         del self.secrets[secret.key]



# import os

# import boto3
# import botocore

# from mind_castle.secret_store_base import SecretStoreBase
# from mind_castle.exceptions import (
#     RetrieveSecretException,
#     CreateSecretException,
#     DeleteSecretException,
#     UpdateSecretException,
# )


# class AWSSecretsManagerSecretStore(SecretStoreBase):
#     """
#     Uses AWS Secrets Manager to store secrets.
#     """

#     store_type = "awssecretsmanager"
#     required_config = [
#         [
#             "MIND_CASTLE_AWS_REGION",
#             "MIND_CASTLE_AWS_ACCESS_KEY_ID",
#             "MIND_CASTLE_AWS_SECRET_ACCESS_KEY",
#         ],
#         ["MIND_CASTLE_AWS_REGION", "MIND_CASTLE_AWS_USE_ENV_AUTH"],
#     ]
#     optional_config = []

#     def __init__(self):
#         if all([os.environ.get(req) for req in self.required_config[0]]):
#             # Configure with secret key
#             self.client = boto3.client(
#                 "secretsmanager",
#                 region_name=os.environ["MIND_CASTLE_AWS_REGION"],
#                 aws_access_key_id=os.environ["MIND_CASTLE_AWS_ACCESS_KEY_ID"],
#                 aws_secret_access_key=os.environ["MIND_CASTLE_AWS_SECRET_ACCESS_KEY"],
#             )
#         else:
#             # Assume the environment is configured with the correct credentials
#             self.client = boto3.client(
#                 "secretsmanager", region_name=os.environ["MIND_CASTLE_AWS_REGION"]
#             )

#     def retrieve_secret(self, secret_details: dict) -> str:
#         key = secret_details.get("key")
#         try:
#             response = self.client.get_secret_value(SecretId=secret_details.get("key"))
#         except botocore.exceptions.ClientError as e:
#             raise RetrieveSecretException(e, key)

#         return response.get("SecretString")

#     def create_secret(self, key: str, value: str) -> dict:
#         # Response is always a success. This throws an exception if it fails
#         request_token = self.get_secret_key()
#         try:
#             response = self.client.create_secret(
#                 Name=key,
#                 ClientRequestToken=request_token,
#                 SecretString=value,
#                 # Other details we could add:
#                 # Description='string',
#                 # KmsKeyId='string',
#                 # SecretBinary=b'bytes',
#                 # Tags=[
#                 #     {
#                 #         'Key': 'string',
#                 #         'Value': 'string'
#                 #     },
#                 # ],
#                 # AddReplicaRegions=[
#                 #     {
#                 #         'Region': 'string',
#                 #         'KmsKeyId': 'string'
#                 #     },
#                 # ],
#                 # ForceOverwriteReplicaSecret=True|False
#             )
#         except botocore.exceptions.ClientError as e:
#             raise CreateSecretException(e)

#         # ARN is not needed but returned just in case. we could also add tags etc to this response
#         return {
#             "mind_castle_secret_type": self.store_type,
#             "request_token": request_token,
#             "key": key,
#             "arn": response.get("ARN"),
#         }

#     def update_secret(self, secret_details: dict, value: str) -> dict:
#         # Response is always a success. This throws an exception if it fails
#         key = secret_details.get("key")
#         request_token = self.get_secret_key()
#         try:
#             response = self.client.put_secret_value(
#                 SecretId=key,
#                 ClientRequestToken=request_token,
#                 SecretString=value,
#             )
#         except botocore.exceptions.ClientError as e:
#             raise UpdateSecretException(e)

#         # ARN is not needed but returned just in case. we could also add tags etc to this response
#         return {
#             "mind_castle_secret_type": self.store_type,
#             "request_token": request_token,
#             "key": key,
#             "arn": response.get("ARN"),
#         }

#     def delete_secret(self, secret_details: dict) -> None:
#         key = secret_details["key"]
#         # Response is always a success. This throws an exception if it fails
#         try:
#             self.client.delete_secret(SecretId=key)
#         except botocore.exceptions.ClientError as e:
#             raise DeleteSecretException(e, key)



# from mind_castle.stores.memory import MemorySecretStore


# def test_create_secret():
#     secret_store = MemorySecretStore()
#     response = secret_store.create_secret("some_secret_value")

#     # Just read the memory from this implementation
#     assert secret_store.secrets[response.key].encrypted_value == "some_secret_value"
#     assert response.mind_castle_secret_type == secret_store.store_type


# def test_retrieve_secret():
#     secret_store = MemorySecretStore()
#     response = secret_store.create_secret("some_secret_value")

#     assert secret_store.retrieve_secret(response) == "some_secret_value"


# def test_update_secret():
#     secret_store = MemorySecretStore()

#     response = secret_store.create_secret("some_secret_value")
#     assert secret_store.retrieve_secret(response) == "some_secret_value"

#     response = secret_store.update_secret(response, "some_updated_value")
#     assert secret_store.retrieve_secret(response) == "some_updated_value"


# def test_delete_secret():
#     secret_store = MemorySecretStore()
#     response = secret_store.create_secret("some_secret_value")

#     secret_store.delete_secret(response)
#     # Just read the memory from this implementation
#     assert response.key not in secret_store.secrets
